/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* FORMATS.CPP                                               */
/*************************************************************/

#ifndef FORMATS_H
#include "formats.h"
#endif

/******************************************* */
/*					     */
/*    Data Management Format Classes         */
/*					     */
/* ***************************************** */

// Sales Transaction Format member functions

void Sales_Transaction_Format::store_class_name
	(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className",
		"Sales_Transaction");
}

void Sales_Transaction_Format::store_attribute_values
	(ofstream& a_stream)
{
	Transaction_Format<Sales_Transaction>::
		store_attribute_values(a_stream);

	String& payment_string =
		pdc_object->get_method_of_payment();
	this->store_tag_value(a_stream, "methodOfPayment",
		payment_string);
}

void Sales_Transaction_Format::restore_attribute_values
	(ifstream& a_stream)
{
	Transaction_Format<Sales_Transaction>::
		restore_attribute_values(a_stream);
	String& payment_method =
		this->restore_tag_value(a_stream, "methodOfPayment");
	pdc_object->set_method_of_payment(payment_method);
}

// Customer Format member functions

void Customer_Format::store_class_name(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className", "Customer");
}

void Customer_Format::store_attribute_values
	(ofstream& a_stream)
{
	String& name_string = pdc_object->get_name();
	String& address_string = pdc_object->get_address();

	this->store_tag_value(a_stream, "name", name_string);
	this->store_tag_value(a_stream, "address", address_string);
}

void Customer_Format::store_connections(ofstream& a_stream)
{
	Transaction* transaction;
	Transaction_List& transactions =
		pdc_object->get_transactions();
	int trans_ID;
	while(!transactions.end_of_list())
		{
		transaction = transactions.current();
		trans_ID = transaction->get_object_ID();
		this->store_tag(a_stream, "transaction");
		this->store_ID(a_stream, trans_ID);
		transactions++;
		}
}

void Customer_Format::restore_connections(ifstream& a_stream,
	Object_Table* an_object_table)
{
	int object_ID;
	while ((this->restore_tag(a_stream, "transaction")) != 0)
		{
		object_ID = this->restore_ID(a_stream);
		Sales_Transaction* transaction =
			(Sales_Transaction*)an_object_table->
				object_at_ID(object_ID);
		pdc_object->connect_to_transaction(*transaction);
		}
}

void Customer_Format::restore_attribute_values(ifstream& a_stream)
{
	String& name =
		this->restore_tag_value(a_stream, "name");
	String& address =
		this->restore_tag_value(a_stream, "address");

	pdc_object->set_name(name);
	pdc_object->set_address(address);
}

// Line Item Format member functions

void Line_Item_Format::store_class_name(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className",
		"Line_Item");
}

void Line_Item_Format::store_attribute_values(ofstream& a_stream)
{
	int line_number = pdc_object->get_line_number();
	int quantity = pdc_object->get_quantity();

	this->store_tag_value(a_stream, "lineNumber",
		String(line_number));
	this->store_tag_value(a_stream, "quantity",
		String(quantity));
}

void Line_Item_Format::store_connections(ofstream& a_stream)
{
	Product& product = pdc_object->get_product();
	int product_ID = product.get_object_ID();
	this->store_tag(a_stream, "product");
	this->store_ID(a_stream, product_ID);
}

void Line_Item_Format::restore_connections(ifstream& a_stream,
	Object_Table* an_object_table)
{
	int object_ID;
	Product* product;
	this->restore_tag(a_stream, "product");
	object_ID = this->restore_ID(a_stream);
	product = (Product*)an_object_table->
			object_at_ID(object_ID);
	pdc_object->connect_to_product(*product);
}

void Line_Item_Format::restore_attribute_values(ifstream& a_stream)
{
	int line_number, quantity;
	String& number_string =
		this->restore_tag_value(a_stream, "lineNumber");
	line_number = number_string.as_integer();
	number_string =
		this->restore_tag_value(a_stream, "quantity");
	quantity = number_string.as_integer();
	pdc_object->set_line_number(line_number);
	pdc_object->set_quantity(quantity);
}

void Product_Format::store_class_name(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className", "Product");
}

void Product_Format::store_attribute_values(ofstream& a_stream)
{
	String& name_string = pdc_object->get_name();
	float price = pdc_object->get_price();

	this->store_tag_value(a_stream, "name", name_string);
	this->store_tag_value(a_stream, "price", String(price));
}


void Product_Format::restore_attribute_values(ifstream& a_stream)
{
	String& name_string = this->restore_tag_value(a_stream, "name");
	String& number_string = this->restore_tag_value(a_stream, "price");
	float price = number_string.as_float();

	pdc_object->set_name(name_string);
	pdc_object->set_price(price);
}

Sales_Domain_Class_Map::Sales_Domain_Class_Map()
{
	formats.add(new Transaction_Format<Transaction>());
	formats.add(new Sales_Transaction_Format());
	formats.add(new Line_Item_Format());
	formats.add(new Customer_Format());
	formats.add(new Product_Format());
	formats.add(new Object_Table_Format());
}

void Sales_Domain_Class_Map::restore_object(int object_ID,
	int class_code, Object_Table* object_table)
{
	PDC_Object* pdc_object;

	// Create the new PDC object based on its class code.
	switch (class_code)
		{
		case SALES_TRANSACTION:
			pdc_object = new Sales_Transaction ();
			break;
		case LINE_ITEM:
			pdc_object = new Line_Item ();
			break;
		case CUSTOMER:
			pdc_object = new Customer ();
			break;
		case PRODUCT:
			pdc_object = new Product ();
			break;
		}

	// Assign the object's identifier.
	pdc_object->set_object_ID(object_ID);

	// Register the object in the object table.
	object_table->register_object(object_ID, pdc_object);
}



