/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* STORAGE2.H                                                */
/*************************************************************/

#include <iostream.h>
#include <fstream.h>
#include "date.h"
#include "time.h"
#include "cc_boole.h"
#include "olist.h"

#ifndef STORAGE2_H
#define STORAGE2_H

const MAX_TABLE_SIZE = 50;

// Forward references
class Object_Table;
class Object_Table_Object_Iterator;
class Object_Tag_Format;
class Class_Map;

typedef Ordered_List<Object_Tag_Format*>
	Object_Tag_Format_List;

class PDC_Object {
public:

	// Constructors/destructors
	PDC_Object() { object_ID = this->get_next_object_ID(); }
	~PDC_Object() { };

	// Accessors
	int get_object_ID(){ return object_ID; }
	void set_object_ID(int new_ID) { object_ID = new_ID; }

	// Data Management
	void register_in_table(Object_Table* an_object_table);
	virtual int get_class_code() = 0;
	void store_using_table(ofstream& a_stream, Object_Table*
		an_object_table);
	void restore_using_table(ifstream& a_stream,
		Object_Table* an_object_table);
	virtual void register_connections(Object_Table* object_table) { }

protected:

	int get_next_object_ID() { return ++next_object_ID; }

	int object_ID;
	static int next_object_ID;
};


class Object_Tag_Format {
public:

	// Constructors/destructors
	Object_Tag_Format() { }
	~Object_Tag_Format() { }

	// Implementors
	// Storing objects
	// Same for all
	void store_defn_start(ofstream& a_stream);
	void store_defn_end(ofstream& a_stream);

	// May be specialized
	virtual void store(ofstream& a_stream);
	// Optional
	virtual void store_connections(ofstream& a_stream) { }

	// Must be implemented by specialized classes
	virtual void store_class_name(ofstream& a_stream) = 0;
	virtual void store_attribute_values(ofstream& a_stream)
		= 0;

	// Restoring objects
	// Same for all
	void read_defn_start(ifstream& a_stream);
	void read_defn_end(ifstream& a_stream);
	String restore_class_name(ifstream& a_stream);

	// May be specialized
	virtual void restore(ifstream& a_stream, Object_Table*
		an_object_table);

	// Optional
	virtual void restore_connections(ifstream& a_stream,
		Object_Table* an_object_table){ }

	// Must be implemented by specialized classes
	virtual void restore_attribute_values(ifstream&
		a_stream) = 0;

	virtual void set_object(void* new_object) = 0;

protected:

	// Supporting member functions
	void store_tag(ofstream& a_stream,
		String tag_name);
	void store_value(ofstream& a_stream,
		String tag_value);
	void store_tag_value(ofstream& a_stream,
		String tag_name,
		String tag_value);
	void store_ID(ofstream& a_stream,
		int object_ID);
	void store_date(ofstream& a_stream, Date& a_date);
	void store_time(ofstream& a_stream, Time& a_time);

	String restore_value(ifstream& a_stream);
	Boolean restore_tag(ifstream& a_stream, String tag_name);
	String restore_tag_value(ifstream& a_stream, String
		tag_name);
	int restore_ID(ifstream& a_stream);
	Date& restore_date(ifstream& a_stream);
	Time& restore_time(ifstream& a_stream);

};

template<class PDC_Type>
class PDC_Object_Format : public Object_Tag_Format
{
public:

	// Constructors/destructors
	PDC_Object_Format() { }
	~PDC_Object_Format() { }

	// Implementors
	virtual void store(ofstream& a_stream);
	virtual void restore(ifstream& a_stream,
		Object_Table* an_object_table);

	void store_object_ID(ofstream& a_stream);
	void restore_object_ID(ifstream& a_stream);

	virtual void set_object(void* new_object)
		{ pdc_object = (PDC_Type*) new_object; }

	// Accessors
	void set_pdc_object(PDC_Type& new_object)
		{ pdc_object = &new_object ; }
	PDC_Type& get_pdc_object()
		{ return *pdc_object; }

protected:

	PDC_Type* pdc_object;
};

template<class PDC_Type>
void PDC_Object_Format<PDC_Type>::store_object_ID
	(ofstream& a_stream)
{
	int object_ID = pdc_object->get_object_ID();
	this->store_tag(a_stream, "objectID");
	this->store_ID(a_stream, object_ID);
}

template<class PDC_Type>
void PDC_Object_Format<PDC_Type>::store(ofstream& a_stream)
{
	this->store_defn_start(a_stream);
	this->store_class_name(a_stream);
	this->store_object_ID(a_stream);
	this->store_attribute_values(a_stream);
	this->store_connections(a_stream);
	this->store_defn_end(a_stream);
}

template<class PDC_Type>
void PDC_Object_Format<PDC_Type>::restore_object_ID(ifstream& a_stream)
{
	if (!(this->restore_tag(a_stream, "objectID")))
		return;
	int object_ID = this->restore_ID(a_stream);
	pdc_object->set_object_ID(object_ID);
}

template<class PDC_Type>
void PDC_Object_Format<PDC_Type>::restore(ifstream&
	a_stream, Object_Table* an_object_table)
{
	this->read_defn_start(a_stream);
	this->restore_class_name(a_stream);
	this->restore_object_ID(a_stream);
	this->restore_attribute_values(a_stream);
	this->restore_connections(a_stream, an_object_table);
	this->read_defn_end(a_stream);
}


class Object_Table {
public:

	// Constructors/destructors
	Object_Table() { table_index = -1; }
	~Object_Table() { }

	// Implementors
	Boolean save_to_file(char* filename);
	void register_object(int object_ID, PDC_Object*
		pdc_object);

	void store(ofstream& a_stream);
	void store_objects(ofstream& a_stream);

	Boolean read_from_file(char* fileneam);
	void restore(ifstream& a_stream);
	void restore_objects(ifstream& a_stream);

	Object_Table_Object_Iterator& get_objects();

	Class_Map* get_class_map() { return class_map; }
	void set_class_map(Class_Map* new_value)
		{ class_map = new_value; }

	void* object_at_ID(int object_ID);

private:

	// Supporting member functions
	Boolean find_ID(int object_ID);

	// Supporting class
	friend class Object_Table_Object_Iterator;

	// Data members
	int object_IDs[MAX_TABLE_SIZE];
	PDC_Object* objects[MAX_TABLE_SIZE];
	Class_Map* class_map;
	int table_index;
};

class Object_Table_Object_Iterator {
public:

	// Constructors/destructors
	Object_Table_Object_Iterator(Object_Table& new_table)
		{ an_object_table = &new_table;
			position = 0; }
	~Object_Table_Object_Iterator() { }

	// Overloaded function call operator
	PDC_Object* operator() ()
		{ if (position <= an_object_table->table_index)
			return an_object_table->objects[position++];
			else return 0;
		}
	void reset() { position = 0; }
	int size ()  {return an_object_table->table_index; }

private:

	// Data members
	Object_Table* an_object_table;
	int position;
};


class Object_Table_Format : public Object_Tag_Format
{
public:
	Object_Table_Format() { }
	~Object_Table_Format() { }
	void store(ofstream& a_stream);
	void restore(ifstream& a_stream,
		Object_Table* new_object_table);

	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream) {}
	void store_table_start(ofstream& a_stream);
	void store_object_IDs(ofstream& a_stream);
	void store_table_end(ofstream& a_stream);
	void store_class_code(ofstream& a_stream, int
		class_code);

	void restore_attribute_values(ifstream& a_stream) { }
	void read_table_start(ifstream& a_stream);
	void restore_object_IDs(ifstream& a_stream,
		Object_Table* some_object_table);
	void read_table_end(ifstream& a_stream);
	int restore_class_code(ifstream& a_stream);

	void set_object(void* new_object)
		{ an_object_table = (Object_Table*)new_object; }

	// Accessors
	void set_object_table(Object_Table& new_object_table)
		{ an_object_table = &new_object_table; }
	Object_Table& get_object_table()
		{ return *an_object_table; }

private:
	Object_Table* an_object_table;
};

class Class_Map {
public:

	// Constructors/destructors
	Class_Map() { }
	virtual ~Class_Map() { }

	// Implementors

	Object_Tag_Format* get_format_for(int class_code);
	Object_Tag_Format* get_format_for_table()
		{return get_format_for(table_class_code());}

	// Can be specialized by derived classes.
	virtual int table_class_code(){return 0;}
	virtual void restore_object(int object_ID, int
		class_code, Object_Table* an_object_table) { }

protected:

	Object_Tag_Format_List formats;
};




#endif