/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* FLOW6.H                                                   */
/*************************************************************/

#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>

#include "string.h"
#include "olist.h"
#include "cc_boole.h"
#include "time.h"
#include "date.h"
#include "arraycol.h"

#ifndef FLOW6_H
#define FLOW6_h

// Forward reference
class Lane;
class Active_Interval;
class Step;

typedef Ordered_List<Lane*> Lane_List;
typedef Ordered_List<Weekday> Weekday_List;
typedef Ordered_List<Active_Interval*> Active_Interval_List;
typedef Ordered_List<Step*> Step_List;

typedef Array_Collection<Step*> Step_Array;
typedef Array_Collection<Lane*> Lane_Array;

class Lane {
public:

	// Constructors/destructors
	Lane(float max_rate);
	Lane(String a_name = "", float max_rate = 0.0);
	~Lane() { }

	// Implementors
	//Rate adjustment
	Boolean adjust_flow_rate(float desired_flow_rate,
		int transition_duration,
		int duration,
		Boolean only_when_requested = false);
	Boolean adjust_flow_rate_to_max(int transition_duration,
	       int duration,
	       Boolean only_when_requested = false);

	void cut_flow_rate_in_half();

	// Safety precautions
	void close_crossing_lanes(int transition_duration);

	// Lane requests
	void clear_requests() { requested = false;}
	void request() { requested = true; }
	void get_any_waiting_requests();

	// Demo functions
	void close();
	void open ();
	void wait(int duration_in_seconds)
		{ delay(duration_in_seconds * 1000); }
	void print_flow_change(float desired_rate, int duration);

	void four_lane_demo(Lane_Array& a_lane_array);

	// Accessors
	float get_current_flow_rate()
		{ return current_flow_rate; }
	void set_current_flow_rate(float new_value)
		{ current_flow_rate = new_value; }

	float get_max_flow_rate() { return max_flow_rate; }
	void set_max_flow_rate(float new_value)
		{ max_flow_rate = new_value; }

	Lane_List& get_crossing_lanes()
		{ return crossing_lanes; }

	void connect_to_crossing_lane(Lane& new_lane);
	Boolean disconnect_with_crossing_lane(Lane& aLane);

	Boolean get_requested() { return requested; }
	void set_requested(Boolean new_value)
		{ requested = new_value; }
	String get_name() { return name; }
	void set_name(String new_name) { name = new_name; }

	// Overloaded operators
	Boolean operator== (Lane& traffic_lane);
	friend ostream& operator<< (ostream& os, Lane& traffic_lane);

private:
	// Simulating
	int next_random_number() { return random(100); }

	// Data members
	float current_flow_rate;
	float max_flow_rate;
	Lane_List crossing_lanes;
	Boolean requested;

	String name;
};


class Step {
public:

	// Constructors/destructors
	Step();
	Step(float flow_rate, int time_to_transition,
		int step_duration, Boolean request_flag = false);
	~Step() { }

	// Implementors
	void run();

	// Accessors
	int get_transition_duration()
		{ return transition_duration; }
	void set_transition_duration(int new_value)
		{ transition_duration = new_value; }

	int get_duration() { return duration; }
	void set_duration(int new_value) { duration =
		new_value; }

	float get_desired_flow_rate() { return
		desired_flow_rate; }
	void set_desired_flow_rate(float new_value);

	Boolean get_only_when_requested()
		{ return only_when_requested; }
	void set_only_when_requested(Boolean new_value)
		{ only_when_requested = new_value; }

	Lane& get_lane() { return *a_lane; }
	void connect_to_lane(Lane& new_lane)
		{ a_lane = &new_lane; }

	// Overloaded operators
	friend ostream& operator<< (ostream& os, Step& aStep);

	// Supporting members
	Boolean desired_flow_rate_is_max();
	void set_desired_flow_rate_to_max();

	// Demo functions
	int demo_step_duration() { return 5; }
	int demo_transition_duration() { return 4; }
	void demo_steps(Step_Array& step_array, Lane* lane1, Lane* lane2);

private:

	// Data members
	int transition_duration;
	int duration;
	float desired_flow_rate;
	Boolean only_when_requested;
	Lane* a_lane;
	static float flow_rate_maximum;
};


class Active_Interval {
public:

	// Constructors/destructors
	Active_Interval();
	Active_Interval(int day_index1, int day_index2,
		int step);
	~Active_Interval() { }

	// Implementors
	Boolean check_if_active_now();

	// Comparing
	Boolean after_time(Time time_of_day);
	Boolean before_time(Time time_of_day);

	// Testing
	Boolean contains_date(Date time_of_day);
	Boolean contains_time(Time time_of_day);
	Boolean contains_day_of_the_week(Weekday weekday_index);

	// Accessors
	Weekday_List& get_weekdays() { return days_of_the_week; }

	Time get_start_time() { return start_time; }
	void set_start_time(Time new_value)
		{ start_time = new_value; }

	Time get_end_time() { return end_time; }
	void set_end_time(Time new_value){ end_time =
		new_value; }

	// Overloaded operators
	friend ostream& operator<< (ostream& os,
		Active_Interval& an_active_interval);

private:

	// Data members
	Weekday_List days_of_the_week;
	Time start_time;
	Time end_time;
};

class Script {
public:

	// Constructors/destructors
	Script();
	~Script() { }

// Implementors
	// Running
	void run();
	void run_next_step();
	void run_step(Step* aStep);

	// Testing
	Boolean check_if_active_now();

	// Stepping
	Step* get_next_step();
	void increment_number_of_steps();

	// Accessors
	Active_Interval_List& get_active_intervals()
		 { return active_intervals; }
	void connect_to_active_interval
		(Active_Interval& new_interval);

	Step_List& get_steps() { return steps; }
	void connect_to_step(Step& aStep);

	int get_step_index() { return step_index; }
	void set_step_index(int new_value)
		{ step_index = new_value; }

	// Demos
	void demo_short_active_interval();
	void start_demo();
	void demo();

	//Demo setup
	void four_lane_demo(Lane_Array& a_lane_array);


private:

	// Data members
	Active_Interval_List active_intervals;
	Step_List steps;
	int step_index;
};





#endif